/**
 * Class PolynomialRootFinder - Implements the bisection method of root
 * finding for polynomials.
 *
 * @author Matt Johnston
 */

public class PolynomialRootFinder {

/**
 * Finds a particular root of a polynomial, based on initial values.
 * The two initial values must evaluate to opposite signs, however they can
 * be in any order.  The root found will between the two values.
 * in any order.
 * @param poly The Polynomial to find a root of.
 * @param pos1 One of the limits of the range to find the root in.
 * @param pos2 The other limit of the range to find the root in.
 * @param precision The precision with which we should go to find the root.
 * A root will be returned when "precision" is greater than the value
 * the polynomial evaluates to, ie the point is close to zero.
 * @return The x position of the root of the polynomial, within pos1 and pos2.
 * If there are multiple roots, the one found first will be returned.
 */
public static double findroot(Polynomial poly,
								double pos1,
								double pos2,
								double precision) {

	if (poly.evaluate(pos1) * poly.evaluate(pos2) > 0) {
		System.out.println("pos1 and pos2 mustn't evaluate to the same sign");
		System.exit(0);
	}

	// left and right are arbitrary, they aren't actually the positions
	double left = pos1;
	double right = pos2;
	double mid;

	while (Math.abs(poly.evaluate(left)) > precision 
		&& Math.abs(poly.evaluate(right)) > precision) {
	
		mid = (left + right) / 2;
		if (poly.evaluate(mid) * poly.evaluate(left) <= 0) 
			right = mid;
		else
			left = mid;

	}

	if (Math.abs(poly.evaluate(left)) <= precision) return left;
	if (Math.abs(poly.evaluate(right)) <= precision) return right;

	// This should never be reached, is only a precaution.
	System.out.println("An error has occurred. Please check your input");
	System.exit(0);
	return 0;
}

}
